import yaml
import base64
import pathlib

from . import config


class ClientBase:
    def __init__(self, token: str, config_path: str) -> None:
        """
        token from class argument or yaml configuration (token) file.
        """
        if token is None and config_path is None:
            raise NameError(
                "Please pass a token or yaml configuration file (token as a key).")
        
        if token is None:
            if not pathlib.Path(config_path).is_file():
                raise ValueError("config_path: File does not exist.")
            with open(config_path, "r") as file:
                token_config: dict = yaml.load(file, Loader=yaml.FullLoader)

            if config.YAML_TOKEN_KEY not in token_config.keys():
                raise KeyError(f"token: token key required in {config_path} file")

            token: str = token_config[config.YAML_TOKEN_KEY]
        try:
            token_str: str = base64.b64decode(token.encode()).decode()
        except (UnicodeError, UnicodeEncodeError, UnicodeDecodeError, UnicodeTranslateError):
            raise UnicodeError(
                "Invalid Token. Please login to LogicPlum platform to get a new token.")
        except:
            raise ValueError("Unknown Error.")

        token_split: str = token_str.split("||")
        if len(token_split) != 3 or token_split[0] not in config.ALLOWED_ENVIRONMENTS.keys():
            raise ValueError(
                "Invalid token. Please login to LogicPlum platform to get a new token.")

        self.token: str = token
        self.endpoint: str = config.ALLOWED_ENVIRONMENTS[token_split[0]]


class Client(ClientBase):
    def __init__(self, token: str = None, config_path: str = None) -> None:
        super().__init__(token=token, config_path=config_path)



import requests
import pandas as pd

from . import config


class LogicPlum:
    def __init__(self, api_key):
        self.api_key = api_key

    def create_project(self, name, description):
        url: str = f"{config.HOST_NAME}{config.PROJECT_CREATE}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        data: dict = {
            'name': name,
            'description': description,
        }

        response = requests.post(url, data=data, headers=headers)
        if response.status_code == 500: return "Server Error"
        if response.status_code != 201: return response.json().get('detail', None)

        return response.json().get('project_id', None)


    def train(self, project_id, df, target):
        url: str = f"{config.HOST_NAME}{config.MODEL_TRAIN}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        json_df = df.to_json(orient='records')

        data: dict = {
            'project_id': project_id,
            'target': target,
            'data': json_df,
        }
 
        response = requests.post(url, data=data, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)

    def train_status(self, project_id):
        url: str = f"{config.HOST_NAME}{config.MODEL_TRAIN_STATUS}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        params: dict = {
            'project_id': project_id,
        }

        response = requests.get(url, params=params, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)

    def model_list(self, project_id):
        url: str = f"{config.HOST_NAME}{config.MODEL_LIST}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        params: dict = {
            'project_id': project_id,
        }

        response = requests.get(url, params=params, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)

    def deployment_list(self, project_id):
        url: str = f"{config.HOST_NAME}{config.DEPLOY_LIST}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        params: dict = {
            'project_id': project_id,
        }

        response = requests.get(url, params=params, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)

    def deploy(self, project_id, model_id):
        url: str = f"{config.HOST_NAME}{config.DEPLOY}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        data: dict = {
            'project_id': project_id,
            'model_id': model_id,
        }
 
        response = requests.post(url, data=data, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)

    def score(self, deployment_id, df):
        url: str = f"{config.HOST_NAME}{config.MODEL_SCORE}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        json_df = df.to_json(orient='records')

        data: dict = {
            'deployment_id': deployment_id,
            'data': json_df,
        }

        response = requests.post(url, data=data, headers=headers)
        if response.status_code == 500: return "Server Error"
        if response.status_code != 200: return response.json().get('detail', None)

        return pd.read_json(response.json().get('prediction', None), orient='records')

    def blueprint(self, project_id, model_id):
        url: str = f"{config.HOST_NAME}{config.MODEL_BLUEPRINT}"

        headers: dict = {
            'Authorization': self.api_key,
            'version': '100',
        }

        data: dict = {
            'project_id': project_id,
            'model_id': model_id,
        }
 
        response = requests.post(url, data=data, headers=headers)
        if response.status_code == 500: return "Server Error"

        return response.json().get('detail', None)



HOST_NAME = "https://staging-api.logicplum.com/"

PROJECT_CREATE = "api/v2/external/project"
MODEL_TRAIN = "api/v2/external/data/training"
MODEL_TRAIN_STATUS = "api/v2/external/data/training"
MODEL_LIST = "api/v2/external/model/list"
DEPLOY = "api/v2/external/deployment"
DEPLOY_LIST = "api/v2/external/deployment/list"
MODEL_SCORE = "api/v2/external/data/scoring"
MODEL_BLUEPRINT = "api/v2/external/blueprint"

ALLOWED_ENVIRONMENTS: dict = {
    "local": "http://127.0.0.1:8000/api/v2/", 
    "sandbox": "https://sb-api.logicplum.com/api/v2/", 
    "staging": "https://staging-api.logicplum.com/api/v2/", 
    "production": "https://api.logicplum.com/api/v2/",
}

YAML_TOKEN_KEY: str = "token"
